"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = watchFiles;
const child_process_1 = require("child_process");
const fs_1 = __importDefault(require("fs"));
const delay_1 = __importDefault(require("../../utils/delay"));
let timeout;
const UPDATE_TIMEOUT = 2000;
function watchFiles(_a) {
    return __awaiter(this, arguments, void 0, function* ({ files, options, }) {
        try {
            for (let i = 0; i < files.length; i++) {
                const file = files[i];
                const filePath = typeof file == "string" ? file : (file === null || file === void 0 ? void 0 : file.path) ? file.path : null;
                const interval = typeof file == "object" ? file.interval : null;
                if (!filePath)
                    continue;
                if (typeof file == "string" && !fs_1.default.existsSync(filePath)) {
                    try {
                        const existingFilePath = files.find((fl) => {
                            if (typeof fl == "string")
                                return fs_1.default.existsSync(fl);
                            if (!fl.host)
                                return fs_1.default.existsSync(fl.path); // TODO handle remote
                        });
                        if (!existingFilePath) {
                            throw new Error("No existing Files for reference");
                        }
                        const fileDirPath = typeof existingFilePath == "string"
                            ? existingFilePath
                            : existingFilePath.path;
                        if (!fs_1.default.existsSync(fileDirPath)) {
                            fs_1.default.mkdirSync(fileDirPath, { recursive: true });
                        }
                        fs_1.default.writeFileSync(filePath, "");
                        if (typeof existingFilePath == "string") {
                            sync({ filePath: existingFilePath, files, options });
                        }
                        else {
                            sync({
                                filePath: existingFilePath.path,
                                files,
                                options,
                            });
                        }
                    }
                    catch (error) {
                        throw new Error(`File Doesn't exist and couldn't be created. Please check if Directory exists.\nERROR => ${error.message}`);
                    }
                }
                if (typeof file == "string" && !fs_1.default.statSync(filePath).isFile()) {
                    throw new Error(`'${filePath}' is not a File!`);
                }
                if (typeof file == "object" && file.host) {
                    // TODO Handle SSH
                }
                else if (typeof file == "string") {
                    sync({ options, filePath, files });
                    yield (0, delay_1.default)();
                    fs_1.default.watchFile(filePath, {
                        interval: interval || 500,
                    }, (curr, prev) => {
                        const INTERVAL = (options === null || options === void 0 ? void 0 : options.interval)
                            ? options.interval
                            : UPDATE_TIMEOUT;
                        clearTimeout(timeout);
                        timeout = setTimeout(() => {
                            sync({ options, filePath, files });
                            process.exit(1);
                        }, INTERVAL);
                    });
                }
            }
        }
        catch (error) {
            console.log("ERROR:", error.message);
            process.exit(0);
        }
    });
}
function sync({ options, filePath, files }) {
    var _a;
    const destFiles = files.filter((fl) => {
        if (typeof fl == "string")
            return fl !== filePath;
        if (fl === null || fl === void 0 ? void 0 : fl.path)
            return fl.path !== filePath;
        return false;
    });
    for (let j = 0; j < destFiles.length; j++) {
        let cmdArray = ["rsync", "-avh"];
        if (options === null || options === void 0 ? void 0 : options.delete) {
            cmdArray.push("--delete");
        }
        if ((_a = options === null || options === void 0 ? void 0 : options.exclude) === null || _a === void 0 ? void 0 : _a[0]) {
            options.exclude.forEach((excl) => {
                cmdArray.push(`--exclude '${excl}'`);
            });
        }
        const dstFl = destFiles[j];
        if (typeof dstFl == "string") {
            if (!fs_1.default.existsSync(dstFl))
                continue;
            if (filePath === dstFl) {
                console.log(`You can't sync the same paths. Please check your configuration and resolve duplicate paths`);
                process.exit(6);
            }
            cmdArray.push(filePath, dstFl);
            const cmd = cmdArray.join(" ");
            console.log(`Running cmd 1 => ${cmd}`);
            (0, child_process_1.execSync)(cmd, {
                stdio: "inherit",
            });
        }
        else if (dstFl.path) {
            if (!dstFl.host && !fs_1.default.existsSync(dstFl.path))
                continue;
            if (filePath === dstFl.path) {
                console.log(`You can't sync the same paths. Please check your configuration and resolve duplicate paths`);
                process.exit(6);
            }
            if (dstFl.host && dstFl.ssh_key && dstFl.user) {
                cmdArray.push("-e", `'ssh -i ${dstFl.ssh_key}'`);
                cmdArray.push(filePath, `${dstFl.user}@${dstFl.host}:${dstFl.path}`);
                const cmd = cmdArray.join(" ");
                (0, child_process_1.execSync)(cmd, {
                    stdio: "inherit",
                });
            }
            else {
                cmdArray.push(filePath, dstFl.path);
                const cmd = cmdArray.join(" ");
                console.log(`Running cmd 2 => ${cmd}`);
                (0, child_process_1.execSync)(cmd, {
                    stdio: "inherit",
                });
            }
        }
    }
}
